/**
 * OWASP CSRFTester
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * Copyright (c) 2007 - The OWASP Foundation
 * 
 * The CSRFTester is published by OWASP under the LGPL. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @created 2007
 */

package org.owasp.csrftester;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.regex.PatternSyntaxException;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.Color;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.owasp.csrftester.report.Report;
import org.owasp.csrftester.report.ReportFactory;

/**
 *
 * @author  jwilliams
 */
public class CSRFTesterUI extends javax.swing.JFrame implements ListSelectionListener, TableModelListener {
    
    private boolean recording = false;
    CSRFTesterModel model = null;
    CSRFTesterListTableModel requestQueryModel = null;
    CSRFTesterListTableModel requestFormModel = null;
    
    /** Creates new form CSRFTesterUI */
    public CSRFTesterUI(CSRFTesterModel model) {
        initComponents();
        this.model = model;
        jtable.setModel( model );
        ListSelectionModel selectionModel = jtable.getSelectionModel();
        selectionModel.addListSelectionListener( this );
        includeRegexField.setText( CSRFTesterModel.DEFAULT_INCLUDE_PATTERN );
        excludeRegexField.setText( CSRFTesterModel.DEFAULT_EXCLUDE_PATTERN );
        requestQueryModel = new CSRFTesterListTableModel( "Query Parameters");
        requestFormModel = new CSRFTesterListTableModel( "Form Parameters");
        requestQueryModel.addTableModelListener( this );
        requestFormModel.addTableModelListener( this );
        requestQueryParams.setModel( requestQueryModel );
        requestFormParams.setModel( requestFormModel );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">                          
    private void initComponents() {
        jPopupMenu1 = new javax.swing.JPopupMenu();
        jMenuItem4 = new javax.swing.JMenuItem();
        jMenuItem5 = new javax.swing.JMenuItem();
        jPopupMenu2 = new javax.swing.JPopupMenu();
        jMenuItem6 = new javax.swing.JMenuItem();
        htmlSaveDialog = new javax.swing.JFileChooser();
        rawFileSaveDialog = new javax.swing.JFileChooser();
        rawFileLoadDialog = new javax.swing.JFileChooser();
        jPanel1 = new javax.swing.JPanel();
        startButton = new javax.swing.JToggleButton();
        titleLabel = new javax.swing.JLabel();
        clearAllButton = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        tableScrollPane = new javax.swing.JScrollPane();
        jtable = new javax.swing.JTable();
        requestEditor = new javax.swing.JPanel();
        requestStep = new javax.swing.JTextField();
        requestPause = new javax.swing.JTextField();
        requestMethod = new javax.swing.JComboBox();
        requestURL = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        requestQueryParams = new javax.swing.JTable();
        jScrollPane2 = new javax.swing.JScrollPane();
        requestFormParams = new javax.swing.JTable();
        jPanel2 = new javax.swing.JPanel();
        includeLabel = new javax.swing.JLabel();
        excludeLabel = new javax.swing.JLabel();
        includeRegexField = new javax.swing.JTextField();
        excludeRegexField = new javax.swing.JTextField();
        excludeResetButton = new javax.swing.JButton();
        includeResetButton = new javax.swing.JButton();
        testButton = new javax.swing.JButton();
        jCheckBox1 = new javax.swing.JCheckBox();
        jLabel1 = new javax.swing.JLabel();
        jRadioButton1 = new javax.swing.JRadioButton();
        jRadioButton2 = new javax.swing.JRadioButton();
        jRadioButton3 = new javax.swing.JRadioButton();
        jRadioButton4 = new javax.swing.JRadioButton();
        jRadioButton5 = new javax.swing.JRadioButton();
        jSeparator1 = new javax.swing.JSeparator();
        statusBar = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        menuFile = new javax.swing.JMenu();
        menuSave = new javax.swing.JMenuItem();
        menuLoad = new javax.swing.JMenuItem();
        menuExit = new javax.swing.JMenuItem();
        menuOptions = new javax.swing.JMenu();
        jMenuItem3 = new javax.swing.JMenuItem();

        jMenuItem4.setText("Delete selected rows");
        jMenuItem4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem4ActionPerformed(evt);
            }
        });

        jPopupMenu1.add(jMenuItem4);

        jMenuItem5.setText("TBD");
        jPopupMenu1.add(jMenuItem5);

        jMenuItem6.setText("Eric");
        jPopupMenu2.add(jMenuItem6);

        htmlSaveDialog.setAcceptAllFileFilterUsed(false);
        htmlSaveDialog.setDialogTitle("Choose directory to store saved CSRF attack");
        htmlSaveDialog.setDialogType(javax.swing.JFileChooser.SAVE_DIALOG);
        htmlSaveDialog.setFileHidingEnabled(true);
        htmlSaveDialog.setFileSelectionMode(javax.swing.JFileChooser.DIRECTORIES_ONLY);
        rawFileSaveDialog.setAcceptAllFileFilterUsed(false);
        rawFileSaveDialog.setDialogTitle("Save project file");
        rawFileSaveDialog.setDialogType(javax.swing.JFileChooser.SAVE_DIALOG);
        rawFileSaveDialog.setFileHidingEnabled(true);
        rawFileLoadDialog.setAcceptAllFileFilterUsed(false);
        rawFileLoadDialog.setDialogTitle("Load saved project file");
        rawFileLoadDialog.setFileHidingEnabled(true);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("OWASP CSRFTester");
        setBackground(new java.awt.Color(255, 255, 255));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                exitHandler(evt);
            }
        });

        startButton.setText("Start Recording");
        startButton.setMaximumSize(new java.awt.Dimension(150, 25));
        startButton.setMinimumSize(new java.awt.Dimension(150, 25));
        startButton.setPreferredSize(new java.awt.Dimension(150, 25));
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });

        titleLabel.setFont(new java.awt.Font("Papyrus", 1, 18));
        titleLabel.setText("OWASP CSRFTester");

        clearAllButton.setText("Clear All");
        clearAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearAllButtonActionPerformed(evt);
            }
        });

        jSplitPane1.setBackground(new java.awt.Color(255, 255, 255));
        jSplitPane1.setDividerLocation(200);
        jSplitPane1.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        tableScrollPane.setBackground(java.awt.SystemColor.info);
        tableScrollPane.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        tableScrollPane.setComponentPopupMenu(jPopupMenu1);
        tableScrollPane.setAutoscrolls(true);
        jtable.setComponentPopupMenu(jPopupMenu1);
        jtable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4", "Title 5"
            }
        ));
        jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_NEXT_COLUMN);
        jtable.setShowVerticalLines(false);
        tableScrollPane.setViewportView(jtable);

        jSplitPane1.setTopComponent(tableScrollPane);

        requestStep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                requestStepActionPerformed(evt);
            }
        });

        requestPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                requestPauseActionPerformed(evt);
            }
        });

        requestMethod.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "GET", "POST", "TRACE" }));
        requestMethod.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                requestMethodActionPerformed(evt);
            }
        });

        requestURL.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                requestURLActionPerformed(evt);
            }
        });

        requestQueryParams.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane1.setViewportView(requestQueryParams);

        requestFormParams.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPane2.setViewportView(requestFormParams);

        javax.swing.GroupLayout requestEditorLayout = new javax.swing.GroupLayout(requestEditor);
        requestEditor.setLayout(requestEditorLayout);
        requestEditorLayout.setHorizontalGroup(
            requestEditorLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, requestEditorLayout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 306, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 306, Short.MAX_VALUE))
            .addGroup(requestEditorLayout.createSequentialGroup()
                .addGap(18, 18, 18)
                .addComponent(requestMethod, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(requestURL, javax.swing.GroupLayout.DEFAULT_SIZE, 536, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, requestEditorLayout.createSequentialGroup()
                .addComponent(requestStep, javax.swing.GroupLayout.DEFAULT_SIZE, 553, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(requestPause, javax.swing.GroupLayout.PREFERRED_SIZE, 59, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        requestEditorLayout.setVerticalGroup(
            requestEditorLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(requestEditorLayout.createSequentialGroup()
                .addGroup(requestEditorLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(requestStep, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(requestPause, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(requestEditorLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(requestURL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(requestMethod, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(requestEditorLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE)
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 92, Short.MAX_VALUE))
                .addContainerGap())
        );
        jSplitPane1.setBottomComponent(requestEditor);

        includeLabel.setText("Include Regex: ");

        excludeLabel.setText("Exclude Regex:");

        includeRegexField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                includeRegexFieldActionPerformed(evt);
            }
        });
        includeRegexField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                includeRegexFieldKeyTyped(evt);
            }
        });

        excludeRegexField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                excludeRegexFieldActionPerformed(evt);
            }
        });
        excludeRegexField.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                excludeRegexFieldKeyTyped(evt);
            }
        });

        excludeResetButton.setText("Reset");
        excludeResetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                excludeResetButtonActionPerformed(evt);
            }
        });

        includeResetButton.setText("Reset");
        includeResetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                includeResetButtonActionPerformed(evt);
            }
        });

        testButton.setText("Generate HTML");
        testButton.setMaximumSize(new java.awt.Dimension(150, 25));
        testButton.setMinimumSize(new java.awt.Dimension(150, 25));
        testButton.setPreferredSize(new java.awt.Dimension(150, 25));
        testButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                testButtonActionPerformed(evt);
            }
        });

        jCheckBox1.setText("Display in Browser");
        jCheckBox1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jCheckBox1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jCheckBox1.setSelected(true);

        jLabel1.setText("Report Type:");

        jRadioButton1.setSelected(true);
        jRadioButton1.setText("Forms");
        jRadioButton1.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButton1.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton1ActionPerformed(evt);
            }
        });

        jRadioButton2.setText("iFrame");
        jRadioButton2.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButton2.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton2ActionPerformed(evt);
            }
        });

        jRadioButton3.setText("IMG");
        jRadioButton3.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButton3.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton3ActionPerformed(evt);
            }
        });

        jRadioButton4.setText("XHR");
        jRadioButton4.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButton4.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton4ActionPerformed(evt);
            }
        });
        
        jRadioButton5.setText("Link");
        jRadioButton5.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        jRadioButton5.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jRadioButton5.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jRadioButton5ActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(excludeLabel)
                    .addComponent(includeLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(includeRegexField, javax.swing.GroupLayout.DEFAULT_SIZE, 473, Short.MAX_VALUE)
                    .addComponent(excludeRegexField, javax.swing.GroupLayout.DEFAULT_SIZE, 473, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(excludeResetButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(includeResetButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addComponent(jLabel1)
                .addGap(14, 14, 14)
                .addComponent(jRadioButton1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jRadioButton2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jRadioButton3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jRadioButton4)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jRadioButton5)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 93, Short.MAX_VALUE)
                .addComponent(jCheckBox1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(testButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(includeLabel)
                    .addComponent(includeResetButton)
                    .addComponent(includeRegexField, javax.swing.GroupLayout.PREFERRED_SIZE, 21, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(0, 0, 0)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(excludeLabel)
                    .addComponent(excludeResetButton)
                    .addComponent(excludeRegexField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 14, Short.MAX_VALUE)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(testButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jCheckBox1)
                    .addComponent(jRadioButton5)
                    .addComponent(jRadioButton4)
                    .addComponent(jRadioButton3)
                    .addComponent(jRadioButton2)
                    .addComponent(jRadioButton1)
                    .addComponent(jLabel1)))
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(titleLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 310, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(clearAllButton, javax.swing.GroupLayout.PREFERRED_SIZE, 120, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(startButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(10, 10, 10))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 620, Short.MAX_VALUE))
                        .addContainerGap())))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {clearAllButton, startButton});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(titleLabel)
                    .addComponent(startButton, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(clearAllButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 361, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        statusBar.setFont(new java.awt.Font("Tahoma", 0, 10));
        statusBar.setForeground(new java.awt.Color(153, 153, 153));
        statusBar.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 4, 4, 1));
        statusBar.setVerticalTextPosition(javax.swing.SwingConstants.TOP);

        menuFile.setText("File");
        menuFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuFileActionPerformed(evt);
            }
        });

        menuSave.setText("Save");
        menuSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuSaveActionPerformed(evt);
            }
        });

        menuFile.add(menuSave);

        menuLoad.setText("Load");
        menuLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuLoadActionPerformed(evt);
            }
        });

        menuFile.add(menuLoad);

        menuExit.setText("Exit");
        menuExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                menuExitActionPerformed(evt);
            }
        });

        menuFile.add(menuExit);

        jMenuBar1.add(menuFile);

        menuOptions.setText("Options");
        jMenuItem3.setText("TBD");
        jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMenuItem3ActionPerformed(evt);
            }
        });

        menuOptions.add(jMenuItem3);

        jMenuBar1.add(menuOptions);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSeparator1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 640, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(statusBar, javax.swing.GroupLayout.DEFAULT_SIZE, 640, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(statusBar, javax.swing.GroupLayout.PREFERRED_SIZE, 17, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pack();
    }// </editor-fold>                        

    private void jRadioButton5ActionPerformed(java.awt.event.ActionEvent evt) {
    	jRadioButton1.setSelected(false);
        jRadioButton2.setSelected(false);
        jRadioButton3.setSelected(false);
        jRadioButton4.setSelected(false);
        jRadioButton5.setSelected(true);
    }   
    
    private void jRadioButton4ActionPerformed(java.awt.event.ActionEvent evt) {                                              
        jRadioButton1.setSelected(false);
        jRadioButton2.setSelected(false);
        jRadioButton3.setSelected(false);
        jRadioButton4.setSelected(true);
        jRadioButton5.setSelected(false);
    }                                             

    private void jRadioButton3ActionPerformed(java.awt.event.ActionEvent evt) {                                              
        jRadioButton1.setSelected(false);
        jRadioButton2.setSelected(false);
        jRadioButton3.setSelected(true);
        jRadioButton4.setSelected(false);
        jRadioButton5.setSelected(false);
    }                                             

    private void jRadioButton2ActionPerformed(java.awt.event.ActionEvent evt) {                                              
        jRadioButton1.setSelected(false);
        jRadioButton2.setSelected(true);
        jRadioButton3.setSelected(false);
        jRadioButton4.setSelected(false);
        jRadioButton5.setSelected(false);
    }                                             

    private void jRadioButton1ActionPerformed(java.awt.event.ActionEvent evt) {
    	jRadioButton1.setSelected(true);
        jRadioButton2.setSelected(false);
        jRadioButton3.setSelected(false);
        jRadioButton4.setSelected(false);
        jRadioButton5.setSelected(false);
    }                                             

    private void exitHandler(java.awt.event.WindowEvent evt) {                             
        if ( model.saveNeeded() ) {
            // FIXME: implement save here
            System.out.println( "SAVE NEEDED" );
        }
        System.exit( 0 );
    }                            

    private void menuLoadActionPerformed(java.awt.event.ActionEvent evt) {                                         
        rawFileLoadDialog.showOpenDialog( this );
        File f = rawFileLoadDialog.getSelectedFile();
        try {
            statusBar.setText( "Loading " + f.getName() );
            model.load( f );
            statusBar.setText( "Loaded " + f.getName() );
        } catch ( IOException e ) {
            statusBar.setText( "Error loading " + f.getName() + " - " + e.getMessage() );
        }
    }                                        

    private void requestPauseActionPerformed(java.awt.event.ActionEvent evt) {                                             
        String value = requestPause.getText();
        int row = jtable.getSelectedRow();
        model.setPause( row, value );
    }                                            

    private void requestMethodActionPerformed(java.awt.event.ActionEvent evt) {                                              
        String value = requestMethod.getSelectedItem().toString();
        int row = jtable.getSelectedRow();
        model.setMethod( row, value );
    }                                             

    private void requestURLActionPerformed(java.awt.event.ActionEvent evt) {                                           
        String value = requestURL.getText();
        String params = ((CSRFTesterListTableModel)requestQueryParams.getModel()).getParams();
        if ( params != null && params.length() > 0 ) value += "?" + params;
        int row = jtable.getSelectedRow();
        model.setURL( row, value );
    }                                          
    
    private void requestStepActionPerformed(java.awt.event.ActionEvent evt) {                                            
        String value = requestStep.getText();
        int row = jtable.getSelectedRow();
        model.setStepName( row, value );
    }                                           
        
    private void excludeResetButtonActionPerformed(java.awt.event.ActionEvent evt) {                                                   
        excludeRegexFieldActionPerformed( new ActionEvent(this.includeRegexField, 0, CSRFTesterModel.DEFAULT_EXCLUDE_PATTERN ));
    }                                                  
    
    private void includeResetButtonActionPerformed(java.awt.event.ActionEvent evt) {                                                   
        includeRegexFieldActionPerformed( new ActionEvent(this.includeRegexField, 0, CSRFTesterModel.DEFAULT_INCLUDE_PATTERN ));
    }                                                  
    
    private void startRecording() {
        recording = true;
        startButton.setText( "Stop Recording" );
        setStatus( false, "Recording started" );
    }
    
    private void stopRecording() {
        recording = false;
        startButton.setText( "Start Recording" );
        startButton.setSelected(false);
        setStatus( false, "Recording stopped" );
    }
    
    public void setStatus( boolean error, String message ) {
        if ( error ) {
            statusBar.setText( "Error: " + message );
            Toolkit.getDefaultToolkit().beep();
        } else {
            statusBar.setText( message );
        }
    }
    
    private void clearAllButtonActionPerformed(java.awt.event.ActionEvent evt) {                                               
        stopRecording();
        model.clear();
        setStatus( false, "All requests cleared" );
    }                                              
    
    private void excludeRegexFieldActionPerformed(java.awt.event.ActionEvent evt) {                                                  
        String pattern = evt.getActionCommand();
        try {
            model.setExcludePattern( pattern );
            setStatus( false, "" );
        } catch(PatternSyntaxException e ) {
            setStatus( true, e.getMessage() );
            return;
        }
        excludeRegexField.setText( pattern );
        excludeRegexField.setBackground( Color.WHITE );
    }                                                 
    
    private void includeRegexFieldActionPerformed(java.awt.event.ActionEvent evt) {                                                  
        String pattern = evt.getActionCommand();
        try {
            model.setIncludePattern( pattern );
            setStatus( false, "" );
        } catch(PatternSyntaxException e ){
            setStatus( true, e.getMessage() );
            return;
        }
        includeRegexField.setText( pattern );
        includeRegexField.setBackground( Color.WHITE );
    }                                                 
    
    private void excludeRegexFieldKeyTyped(java.awt.event.KeyEvent evt) {                                           
        char c = evt.getKeyChar();
        if ( c != evt.VK_ENTER ) excludeRegexField.setBackground( Color.YELLOW );
    }                                          
    
    private void includeRegexFieldKeyTyped(java.awt.event.KeyEvent evt) {                                           
        char c = evt.getKeyChar();
        if ( c != evt.VK_ENTER ) includeRegexField.setBackground( Color.YELLOW );
    }                                          
    
    private void deleteSelectedRows() {
        recording = false;
        startButton.setText( "Start Recording" );
        while ( jtable.getSelectedRowCount() > 0 ) {
            int row = jtable.getSelectedRow();
            model.removeRow( row );
        }
        setStatus( false, "Selected requests deleted" );
    }
    
    private void jMenuItem4ActionPerformed(java.awt.event.ActionEvent evt) {                                           
        deleteSelectedRows();
    }                                          
    
    private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {                                           
        System.out.println( "TBD" );
    }                                          
    
    private void menuExitActionPerformed(java.awt.event.ActionEvent evt) {                                         
        if ( model.saveNeeded() ) {
            // FIXME: implement save here
            System.out.println( "SAVE NEEDED" );
        }
        System.exit( 0 );
    }                                        
    
    private void menuSaveActionPerformed(java.awt.event.ActionEvent evt) {                                         
        rawFileSaveDialog.showSaveDialog( this );
        File f = rawFileSaveDialog.getSelectedFile();
        try {
            statusBar.setText( "Saving " + f.getName() );
            model.save( f );
            statusBar.setText( "Saved " + f.getName() );
        } catch ( IOException e ) {
            statusBar.setText( "Error saving " + f.getName() + " - " + e.getMessage() );
        }
    }                                        
    
    private void menuFileActionPerformed(java.awt.event.ActionEvent evt) {                                         
    }                                        
    
    /** Recording button handler **/
    private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {                                            
        if ( recording ) stopRecording();
        else startRecording();
    }                                           
    
    /** Test button handler **/
    private void testButtonActionPerformed(java.awt.event.ActionEvent evt) {                                           
        try {
        	
        	if(isLinkReport())
        	{
        		String linkText = JOptionPane.showInputDialog(this, "What text would you like this link to display?");
        		
        		model.setLinkText(linkText);
        	}
        	
            htmlSaveDialog.showSaveDialog( this );
            File dir = htmlSaveDialog.getSelectedFile();
            
            /**
             * If 'dir' is null, then we assume the user hit the 'cancel' button.
             */
            if(dir != null) {
            	statusBar.setText( "Saving HTML test file " + dir.getName() );
            	int reportType = getReportType();
            	ReportFactory factory = ReportFactory.getInstance();
            	Report report = factory.getReport(reportType);
            	String url = report.generateHtml(model, dir);
            	
            	if(showInBrowser()) {
            		statusBar.setText("Launching newly created HTML file in browser");
            		BrowserLaunch.openURL(url);
            	}
            	
            	statusBar.setText( "HTML test file saved to " + dir.getName() );
            } else {
            	statusBar.setText("User operation cancelled");
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }
    
    private boolean showInBrowser() {
    	return jCheckBox1.isSelected();
    }
    
    private int getReportType() {
    	int reportType = ReportFactory.FORMS;
    	
    	if(jRadioButton1.isSelected()) {
    		reportType = ReportFactory.FORMS;
    	} else if(jRadioButton2.isSelected()) {
    		reportType = ReportFactory.FRAME;
    	} else if(jRadioButton3.isSelected()) {
    		reportType = ReportFactory.IMG;
    	} else if(jRadioButton4.isSelected()) {
    		reportType = ReportFactory.XHR;
    	} else if(jRadioButton5.isSelected()) {
    		reportType = ReportFactory.LINK;
    	}
    	
    	return reportType;
    }
    
    public boolean isRecording() {
        return recording;
    }
    
    public void valueChanged( ListSelectionEvent event ) {
        if( event.getSource() == jtable.getSelectionModel() && event.getFirstIndex() >= 0 ) {
            int row = jtable.getSelectedRow();
            if ( row < 0 || row > jtable.getRowCount() ) return;
            this.statusBar.setText( "Moving to row " + row );
            
            this.requestStep.setText( model.getStepName( row ) );
            this.requestPause.setText( ""+model.getPause( row ) );
            
            this.requestMethod.setSelectedItem( model.getMethod( row ) );
            String url = model.getURL( row );
            String query = "";
            int index = url.indexOf( '?' );
            if ( index != -1 ) {
                query = url.substring( index + 1 );
                url = url.substring( 0, index );
            }
            this.requestURL.setText( url );
            
            String[] qsplit = query.split( "&" );
            updateListModel( (CSRFTesterListTableModel)requestQueryParams.getModel(), qsplit );
            
            String[] fsplit = model.getParameters( row ).split( "&" );
            updateListModel( (CSRFTesterListTableModel)requestFormParams.getModel(), fsplit );
        }
    }
    
    private void updateListModel( CSRFTesterListTableModel lm, String[] values ) {
        while( lm.getRowCount() > 0 ) lm.removeRow( 0 );
        for( int i = 0; i<values.length; i++ ) {
            String[] row = { values[i] };
            lm.addRow( row );
        }
        String[] blank = { "" };
        lm.addRow( blank );  // add an empty row at the end of the list for adding new values
    }
    
    /* Called whenever either the query or form parameter tables change, so update them both */
    public void tableChanged(TableModelEvent e) {
        int row = jtable.getSelectedRow();
        if ( e.getSource() == requestFormModel ) {
            model.setParameters( row, ((CSRFTesterListTableModel)requestFormParams.getModel()).getParams() );
        } else if ( e.getSource() == requestQueryModel ) {
            String url = requestURL.getText();
            String params = ((CSRFTesterListTableModel)requestQueryParams.getModel()).getParams();
            if ( params != null && params.length() > 0 ) url += "?" + params;
            model.setURL( row, url );
        }
    }
    
    private boolean isLinkReport()
    {
    	return jRadioButton5.isSelected();
    }
    
    // Variables declaration - do not modify                     
    private javax.swing.JButton clearAllButton;
    private javax.swing.JLabel excludeLabel;
    private javax.swing.JTextField excludeRegexField;
    private javax.swing.JButton excludeResetButton;
    private javax.swing.JFileChooser htmlSaveDialog;
    private javax.swing.JLabel includeLabel;
    private javax.swing.JTextField includeRegexField;
    private javax.swing.JButton includeResetButton;
    private javax.swing.JCheckBox jCheckBox1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem3;
    private javax.swing.JMenuItem jMenuItem4;
    private javax.swing.JMenuItem jMenuItem5;
    private javax.swing.JMenuItem jMenuItem6;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPopupMenu jPopupMenu1;
    private javax.swing.JPopupMenu jPopupMenu2;
    private javax.swing.JRadioButton jRadioButton1;
    private javax.swing.JRadioButton jRadioButton2;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JRadioButton jRadioButton4;
    private javax.swing.JRadioButton jRadioButton5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTable jtable;
    private javax.swing.JMenuItem menuExit;
    private javax.swing.JMenu menuFile;
    private javax.swing.JMenuItem menuLoad;
    private javax.swing.JMenu menuOptions;
    private javax.swing.JMenuItem menuSave;
    private javax.swing.JFileChooser rawFileLoadDialog;
    private javax.swing.JFileChooser rawFileSaveDialog;
    private javax.swing.JPanel requestEditor;
    private javax.swing.JTable requestFormParams;
    private javax.swing.JComboBox requestMethod;
    private javax.swing.JTextField requestPause;
    private javax.swing.JTable requestQueryParams;
    private javax.swing.JTextField requestStep;
    private javax.swing.JTextField requestURL;
    private javax.swing.JToggleButton startButton;
    private javax.swing.JLabel statusBar;
    private javax.swing.JScrollPane tableScrollPane;
    private javax.swing.JButton testButton;
    private javax.swing.JLabel titleLabel;
    // End of variables declaration                   
    
}
