/**
 * OWASP CSRFTester
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * Copyright (c) 2007 - The OWASP Foundation
 * 
 * The CSRFTester is published by OWASP under the LGPL. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @created 2007
 */

package org.owasp.csrftester;

import java.util.regex.Pattern;

/**
 * FIXME: This attempt failed for being too complex.  The idea was to mark buttons and
 * links that are actually clicked with a special tag so that CSRFTester can know which ones got selected
 */
public class LinkTransformer {
    
    public final static Pattern FORM_PATTERN = Pattern.compile("(?i)</form>");
    public final static Pattern SKIPPABLE_PATTERN = Pattern.compile( CSRFTesterModel.DEFAULT_EXCLUDE_PATTERN );
    
    // token to use to mark forms and URLs that will be captured by CSRFTester when clicked
    public final static String CSRFTESTER = "CSRFTESTER";
    
    private String errorPage = null;
        

    public static String transform(String original) {
        String content = buildFormParameters( original );
        content = buildLinkParameters( content );
        return content;
    }
        
    private static String buildFormParameters(String content) {
        return FORM_PATTERN.matcher(content).replaceAll("<input type=\"hidden\" name=\"" + CSRFTESTER + "\">\n</form>");
    }
    
    private static String buildLinkParameters(String content) {
        content = buildLinkParameters(content, "<a", "href=");
        // content = buildLinkParameters(content, "src=\"");       
        // content = buildLinkParameters(content, "<form", "action=");       
        return content;
    }
    
    private static String buildLinkParameters(String content, String tag, String attr) {
        StringBuffer buffer = new StringBuffer();
        int i=0;
        int length = content.length();
        int tagEnd = 0;
        
        while(i < length) {
            int tagStart = content.toLowerCase().indexOf(tag, i);
            if ( tagStart != -1 ) {
                buffer.append( content.substring( tagEnd + 1, tagStart ) );
                tagEnd = content.toLowerCase().indexOf( '>', tagStart + 1 );
                if ( tagEnd != -1 ) {
                    String tagContent = content.substring( tagStart, tagEnd + 1);
                    System.out.println( "*" + tagContent );
                    int attrStart = tagContent.toLowerCase().indexOf( attr );
                    if ( attrStart != -1 ) {
                        int attrEnd = tagContent.toLowerCase().indexOf( '\"', attrStart  + attr.length() + 1 ); // skip initial quote
                        if ( attrEnd == -1 ) {
                            attrEnd = tagContent.toLowerCase().indexOf( ' ', tagStart + 1 );
                        }
                        if ( attrEnd == -1 ) {
                            attrEnd = tagContent.length() -1;
                        }
                        if ( attrEnd != -1 ) {
                            String attrContent = tagContent.substring( attrStart + attr.length(), attrEnd );
                            System.out.println( "    AC:" + attrContent );
                            String newAttrContent = attrContent + ( attrContent.contains( "?" ) ? "&" : "?" ) + CSRFTESTER;
                            tagContent = tagContent.substring( 0, attrStart + attr.length() ) + newAttrContent + tagContent.substring( attrEnd );
                            System.out.println( "        -> " + tagContent );
                        }
                    }
                    buffer.append( tagContent );
                    i = tagEnd + 1;
                }
                else {
                    String update = content.substring( i );
                    buffer.append( update );
                    i+= update.length();
                }
            }
            else {
                String update = content.substring( i );
                buffer.append( update );
                i+= update.length();
            }
        }
        return buffer.toString();
    }
    
    public static void main( String[] args )
    {
        String test1 = "blahc  alsdkfj<a name=\"test\">lkwej  f <a href=test?one></a> asdlfkj<form action=\"abc://test\" " +
                "blue=blue>test</form><form test=\"a\" action=\"bang\"> test" +
                "<a class=q href=\"http://froogle.google.com/frghp?ie=UTF-8&oe=UTF-8&hl=en&tab=wf\" onclick=\"return qs(this)\"> test 123";
        System.out.println( "*********A href\n" + test1 );
        System.out.println( "*********A href\n" + buildLinkParameters(test1, "<a", "href="));
        // System.out.println( "*********FORM action\n" + buildLinkParameters(test1, "<form", "action="));       
    }
    
}
