/**
 * OWASP CSRFTester
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * Copyright (c) 2007 - The OWASP Foundation
 * 
 * The CSRFTester is published by OWASP under the LGPL. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @created 2007
 */

package org.owasp.csrftester.report;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.net.URLEncoder;

import org.owasp.csrftester.CSRFTesterModel;

public class IMGReport extends ReportAdapter
{
	/**
	 * DEVNOTE: This report only appears to work on IE
	 */
	protected IMGReport()
	{
		
	}
	
	public String generateHtml(CSRFTesterModel model, File dir) throws IOException
    {
		String[] locations = getLocations(model);
		String html = getImgHtml(locations);
		String content = getContent("/resources/IMGTemplate.html");
		
		content = content.replace("<%ATTACK%>", html);
		writeFile(dir, "index.html", content);
    	return "file://" + dir.getPath() + "/index.html";
    }
	
	private String getImgHtml(String[] locations)
	{
		StringBuffer sb = new StringBuffer();
		
		for(int i=0; i<locations.length; i++)
		{
			sb.append("<img src=\"" + locations[i] + "\" width=\"0\" height=\"0\" border=\"0\"/>\r\n");
		}
		
		return sb.toString();
	}
	
	private String[] getLocations(CSRFTesterModel model) throws IOException
	{
		ArrayList<String> array = new ArrayList<String>();
		final char q = '?';
		final char amp = '&';
		
		for(int i=0; i<model.getRowCount(); i++)
		{
			StringBuffer sb = new StringBuffer();
			String url = model.getURL(i);
			String params = model.getParameters(i);
			
			sb.append(url);
			
			if(url.indexOf(q) != -1)
			{
				sb.append(amp);
			}
			else
			{
				sb.append(q);
			}
			
			sb.append(params);
			
			array.add(sb.toString());
		}
		
		String[] locations = new String[array.size()];
		return array.toArray(locations);
	}
}
