/**
 * OWASP CSRFTester
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * Copyright (c) 2007 - The OWASP Foundation
 * 
 * The CSRFTester is published by OWASP under the LGPL. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @created 2007
 */

package org.owasp.csrftester.report;

import java.util.Map;
import java.util.HashMap;

public final class ReportFactory
{
	public final static int FORMS = 0;
	
	public final static int XHR = 1;
	
	public final static int FRAME = 2;
	
	public final static int IMG = 3;
	
	public final static int LINK = 4;
	
	private static ReportFactory instance = new ReportFactory();
	
	private Map<Integer, String> reports = null;
	
	private ReportFactory()
	{
		final String pg = "org.owasp.csrftester.report.";
		reports = new HashMap<Integer, String>();
		
		reports.put(FORMS, pg + "FormsReport");
		reports.put(XHR, pg + "XHRReport");
		reports.put(FRAME, pg + "FrameReport");
		reports.put(IMG, pg + "IMGReport");
		reports.put(LINK, pg + "LinkReport");
	}
	
	public static ReportFactory getInstance()
	{
		return instance;
	}
	
	public Report getReport(int id)
	{
		Report report = null;
		String s = reports.get(id);
		
		if(s != null)
		{
			report = getReport(s);
		}
		
		return report;
	}
	
	private Report getReport(String s)
	{
		Report report = null;
		
		try
		{
			Class<?> c = Class.forName(s);
			Object o = c.newInstance();
			
			if(o instanceof Report)
			{
				report = (Report)o;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		return report;
	}
}
