/**
 * OWASP CSRFTester
 * 
 * This file is part of the Open Web Application Security Project (OWASP)
 * Copyright (c) 2007 - The OWASP Foundation
 * 
 * The CSRFTester is published by OWASP under the LGPL. You should read and accept the
 * LICENSE before you use, modify, and/or redistribute this software.
 * 
 * @author <a href="http://www.aspectsecurity.com">Aspect Security</a>
 * @created 2007
 */

package org.owasp.csrftester.report;

import java.io.*;

import org.owasp.csrftester.BrowserLaunch;
import org.owasp.csrftester.CSRFTesterModel;

/**
 *
 * @author jwilliams
 */
public final class XHRReport extends ReportAdapter {

    /**
     * DEVNOTE: This report works for IE (if you 'Allow Blocked Content') but fails in FireFox. FireFox gets a 'Permission Denied to XMLHttpRequest.open.'
     */
    protected XHRReport()
    {
    	
    }
    
    public String generateHtml(CSRFTesterModel model, File dir) throws IOException
    {
    	generateMasterHtml(model, dir);
    	return "file://" + dir.getPath() + "/index.html";
    }
    
    private void generateMasterHtml( CSRFTesterModel model, File dir ) throws IOException
    {
        String template = getContent( "/resources/XHRTemplate.html" );
        StringBuffer pauses = new StringBuffer();
        StringBuffer methods = new StringBuffer();
        StringBuffer urls = new StringBuffer();
        StringBuffer params = new StringBuffer();
        int count = model.getRowCount();
        
        for ( int i=0; i < count; i++ )
        {
            pauses.append( "\"" + model.getPause(i) + "\"" );
            methods.append( "\"" + model.getMethod(i) + "\"" );
            urls.append( "\"" + model.getURL(i) + "\"" );
            params.append( "\"" + model.getParameters(i) + "\"" );
            if ( i != count-1 ) {
                pauses.append( "," );
                methods.append( "," );
                urls.append( "," );
                params.append( "," );
            }
        }
        String html = template;
        html = html.replace( "<%COUNT%>", String.valueOf(count) );
        html = html.replace( "<%PAUSES%>", pauses );
        html = html.replace( "<%METHODS%>", methods );
        html = html.replace( "<%URLS%>", urls );
        html = html.replace( "<%PARAMS%>", params );

        writeFile( dir, "index.html", html );
    }
}